/*
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/*
 *  cube.c
 *  This program demonstrates a single modeling transformation,
 *  glScalef() and a single viewing transformation, gluLookAt().
 *  A wireframe cube is rendered.
 */

#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>

static double x_loc = 0.0, y_loc = 0.0, z_loc = 5.0, x_rot_mouse = 0.0, y_rot_mouse = 0.0;

GLboolean smooth = GL_TRUE;
GLboolean lighting = GL_TRUE;
static GLfloat blue[4] = {0.2, 0.2, 1.0, 0.5};
//static GLfloat red[4] = {1.0, 0.2, 0.2, 0.5};
static GLfloat green[4] = {0.2, 1.0, 0.2, 0.5};


void InitMaterials(void)
{
//	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
//	glLightModelfv(GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);  

//	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, front_mat_shininess);
//	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, front_mat_specular);
//	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, front_mat_diffuse);

}


void init(void) 
{
	glClearColor (0.0, 0.0, 0.0, 0.0);
	glShadeModel (GL_FLAT);
	glEnable(GL_DEPTH_TEST);		// corrects for depth
	glScalef (1.0, 1.0, 1.0);		// an axis independent scaling 

	InitMaterials();
}


void display(void)
{
//	GLfloat knots[8] = {0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0};
	
	glLoadIdentity ();		// transform matrix clear

	glClear (GL_COLOR_BUFFER_BIT);
	glColor3f (1.0, 1.0, 1.0);
						// viewpoint setup
	gluLookAt (z_loc, y_loc, x_loc, 0, 0, 0, 0, 1, 0);

	glPushMatrix();			// ?

	glRotatef(x_rot_mouse, 0, 1, 0);
	glRotatef(y_rot_mouse, 0, 0, 1);

	glLineWidth(1);
	glutWireCube (1.0);			// a blank cube, at the origin

	glColor3f(1.0, 1.0, 0);			//set color to yellow

	glBegin(GL_POINTS);			// write points
		glVertex3f(0, 0, 0);
		glVertex3f(0.25, 0.25, 0.25);
		glVertex3f(0.25, 0.25, -0.25);
		glVertex3f(0.25, -0.25, 0.25);
		glVertex3f(0.25, -0.25, -0.25);
		glVertex3f(-0.25, 0.25, 0.25);
		glVertex3f(-0.25, 0.25, -0.25);
		glVertex3f(-0.25, -0.25, 0.25);
		glVertex3f(-0.25, -0.25, -0.25);
	glEnd();

	glEnable(GL_LIGHTING);

	glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION,GL_DIFFUSE);

//	glColor4f(1,0,1, 0.25);

	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, green);   

	glBegin(GL_TRIANGLES);			//every three data points
						//define a triangle.
		glVertex3f(0, -0.5, 0);
		glVertex3f(0.5, 0, 0);
		glVertex3f(0, 0, 0.5);

		glVertex3f(0, 0.5, 0);
		glVertex3f(0.5, 0, 0);
		glVertex3f(0, 0, -0.5);

//		glColor4f(0, 1, 0, 0.25);

		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, blue);   

		glVertex3f(0, -0.5, 0);
		glVertex3f(0.5, 0, 0);
		glVertex3f(0, 0, -0.5);

		glVertex3f(0, 0.5, 0);
		glVertex3f(-0.5, 0, 0);
		glVertex3f(0, 0, -0.5);

	glEnd();

	glDisable(GL_LIGHTING);

	glColor3f(0,1,1);			//set color to cyan

	glLineWidth(2);

	glBegin(GL_LINES);			//write lines
		glVertex3f(0, -0.5, 0);
		glVertex3f(-0.5, 0, 0);

		glVertex3f(0, -0.5, 0);
		glVertex3f(0.5, 0, 0);

		glVertex3f(0, -0.5, 0);
		glVertex3f(0, 0, -.5);

		glVertex3f(0, -0.5, 0);
		glVertex3f(0, 0, 0.5);

		glVertex3f(0, 0.5, 0);
		glVertex3f(-0.5, 0, 0);

		glVertex3f(0, 0.5, 0);
		glVertex3f(0.5, 0, 0);

		glVertex3f(0, 0.5, 0);
		glVertex3f(0, 0, -.5);

		glVertex3f(0, 0.5, 0);
		glVertex3f(0, 0, 0.5);

		glVertex3f(0, 0, 0.5);
		glVertex3f(-0.5, 0, 0);

		glVertex3f(0, 0, -0.5);
		glVertex3f(0.5, 0, 0);

		glVertex3f(-0.5, 0, 0);
		glVertex3f(0, 0, -0.5);

		glVertex3f(0.5, 0, 0);
		glVertex3f(0, 0, 0.5);

	glEnd();         

	glPopMatrix();
	glFlush ();
}

void reshape (int w, int h)
{
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	glFrustum (-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
	glMatrixMode (GL_MODELVIEW);
}

/* ARGSUSED1 */
void keyboard(unsigned char key, int x, int y)
{
	switch (key) {
		case 'a': 
			x_loc += 0.25;	//slide right
			glutPostRedisplay();
			break;
		case 'd':
			x_loc -= 0.25;	//slide left
			glutPostRedisplay();
			break;
		case 'x':		//slide up
			y_loc -= 0.25;
			glutPostRedisplay();
			break;
		case 'w':		//slide left
			y_loc += 0.25;
			glutPostRedisplay();
			break;
		case 'e':		//slide up
			z_loc -= 0.25;
			glutPostRedisplay();
			break;
		case 'z':		//slide left
			z_loc += 0.25;
			glutPostRedisplay();
			break;
		case 27:		//quit program
			exit(0);
			break;
		case 'q':		//quit program
			exit(0);
			break;
	}
}

static void MouseMove( int x, int y )
{
   x_rot_mouse = y;
   y_rot_mouse = x;
   glutPostRedisplay();
}            

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
	glutInitWindowSize (500, 500); 
	glutInitWindowPosition (100, 100);

	if (glutCreateWindow("Isosurface") <= 0)	//name that window
		exit(0);				//quit if no window

	glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH );	init ();

	glutDisplayFunc(display); 
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);

	glutPassiveMotionFunc( MouseMove );      

	glutMainLoop();
	return 0;
}
