/*
 * Copyright (c) 1999 Jeff DeMaagd, GPL copyright.
 
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

#include "readdxf.h"

extern FILE *file_in, *file_out;
int line_count, number;

static GLuint Data = 1;

/* ********************************************************************** */
void read_dxf_file (char *file_input)
{
	file_in = fopen (file_input,"r");
	if (!file_in) {
		printf("  I couldn't read the input file: %s\n", file_input);
		exit(1);
	}

	read_dxf();
}

/************************************************************************/
void read_dxf ()
/* this is where more fun stuff happens.  I will try to read a DXF file
for important or pertainent data */
{
	extern int line_count, number;	
	int face_count = 0;
	char record[32];
	struct quad q1;
	arc_t a1;
	point_t p1;
	line_seg_t ls1;
	vector_t temp_vec1, temp_vec2, norm_vec;

	line_count = 0;
	number= -1;	

	printf ("Entering read_dxf\n");
	if ((file_in == NULL))	{
	//	if (file_in == NULL)	{
			printf ("Error opening input file.\n");
			exit(0);
		} else {
	
		fscanf (file_in, "%s\n", record);
		line_count++;

		while (0 != strcmp ("ENTITIES", record))
		{	/* fast forward to the ENTITIES section */
			fscanf (file_in, "%s\n", record);
			line_count++;
		}

		fscanf (file_in, "%d", &number); /* 0 */
		fscanf (file_in, "%s", record);
		line_count += 2;

		glNewList(Data, GL_COMPILE);

//			glEnable(GL_NORMALIZE);

		while ((number == 0) && ( 0 != strcmp ("ENDSEC", record)))	{
			if (0 == strcmp ("LINE", record))	{
				printf(" a line.\n");
				glBegin(GL_LINES);
					ls1 = read_dxf_line ();
					glVertex3f ( ls1.p1.x, ls1.p1.y, ls1.p1.z);
					glVertex3f ( ls1.p2.x, ls1.p2.y, ls1.p2.z);
				glEnd();
			} else if (0 == strcmp ("POINT", record))	{
				p1 = read_dxf_point ();
				printf(" a point.\n");
			} else if (0 == strcmp ("ARC", record))	{
				a1 = read_dxf_arc ();
				printf(" an arc.\n");
			} else if (0 == strcmp ("3DFACE", record))	{
				q1 = read_dxf_3dface ();
				face_count++;

/*				printf("(%7.3f %7.3f %7.3f)\n(%7.3f %7.3f %7.3f)\n(%7.3f %7.3f %7.3f)\n(%7.3f %7.3f %7.3f)\n", 
					q1.p1.x, q1.p1.y, q1.p1.z,
					q1.p2.x, q1.p2.y, q1.p2.z,
					q1.p3.x, q1.p3.y, q1.p3.z,
					q1.p4.x, q1.p4.y, q1.p4.z);
				cut_arc (a1, part_clearance, RAMP); */

				if     ((q1.p3.x != q1.p4.x) || 
					(q1.p3.y != q1.p4.y) || 
					(q1.p3.z != q1.p4.z))	{
					glBegin(GL_QUADS);
						temp_vec1 = make_vector ( q1.p1, q1.p2 );
						temp_vec2 = make_vector ( q1.p1, q1.p3 );
						norm_vec = cross_product ( temp_vec1, temp_vec2 );
						norm_vec = unit_product ( norm_vec );

						glNormal3f ( norm_vec.i, norm_vec.j, norm_vec.k );
						glVertex3f ( q1.p1.x, q1.p1.y, q1.p1.z);
						glVertex3f ( q1.p2.x, q1.p2.y, q1.p2.z);
						glVertex3f ( q1.p3.x, q1.p3.y, q1.p3.z);
						glVertex3f ( q1.p4.x, q1.p4.y, q1.p4.z);
					glEnd();
				} else {
					glBegin(GL_TRIANGLES);
						temp_vec1 = make_vector ( q1.p1, q1.p2 );
						temp_vec2 = make_vector ( q1.p1, q1.p3 );
						norm_vec = cross_product ( temp_vec1, temp_vec2 );
						norm_vec = unit_product ( norm_vec );

						glNormal3f ( norm_vec.i, norm_vec.j, norm_vec.k );
						glVertex3f(q1.p1.x, q1.p1.y, q1.p1.z);
						glVertex3f(q1.p2.x, q1.p2.y, q1.p2.z);
						glVertex3f(q1.p3.x, q1.p3.y, q1.p3.z);
					glEnd();
				}

				glBegin(GL_LINE_STRIP);
					glVertex3f(q1.p1.x, q1.p1.y, q1.p1.z);
					glVertex3f(q1.p2.x, q1.p2.y, q1.p2.z);
					glVertex3f(q1.p3.x, q1.p3.y, q1.p3.z);
					glVertex3f(q1.p4.x, q1.p4.y, q1.p4.z);
					glVertex3f(q1.p1.x, q1.p1.y, q1.p1.z);
				glEnd();

			}

			number = 1; 		/* priming my skip engine */
			while (number != 0) {  /* skip to the next object */
				fscanf (file_in, "%d\n", &number); /* 0 */
				fscanf (file_in, "%s\n", record);
				line_count += 2;
			}

		}

//		glDisable(GL_NORMALIZE);

		glEndList();

		printf ("Read in %i lines and %i face(s)\n", line_count, face_count);

		fclose(file_in);
	}
} /* read_dxf */

/************************************************************************/

struct quad read_dxf_3dface (void)
{

	int number;
	char record[32];
	struct quad q1;

	extern int line_count;

	fscanf (file_in, " %d", &number); /* 10 */
	line_count++;

	while (number != 10)	{	
		fscanf (file_in, "%s\n", record);
		fscanf (file_in, " %d\n", &number); /* 10 */
		line_count += 2;
	}

	fscanf (file_in, "%lf\n", &q1.p1.x);
	line_count++;

	fscanf (file_in, " %d\n", &number); /* 20 */
	fscanf (file_in, "%lf\n", &q1.p1.y);

	fscanf (file_in, " %d\n", &number); /* 30 */
	fscanf (file_in, "%lf\n", &q1.p1.z);

	fscanf (file_in, " %d\n", &number); /* 11 */
	fscanf (file_in, "%lf\n", &q1.p2.x);

	fscanf (file_in, " %d\n", &number); /* 21 */
	fscanf (file_in, "%lf\n", &q1.p2.y);

	fscanf (file_in, " %d\n", &number); /* 31 */
	fscanf (file_in, "%lf\n", &q1.p2.z);
		
	fscanf (file_in, " %d\n", &number); /* 12 */
	fscanf (file_in, "%lf\n", &q1.p3.x);

	fscanf (file_in, " %d\n", &number); /* 22 */
	fscanf (file_in, "%lf\n", &q1.p3.y);

	fscanf (file_in, " %d\n", &number); /* 32 */
	fscanf (file_in, "%lf\n", &q1.p3.z);
		
	fscanf (file_in, " %d\n", &number); /* 13 */
	fscanf (file_in, "%lf\n", &q1.p4.x);

	fscanf (file_in, " %d\n", &number); /* 23 */
	fscanf (file_in, "%lf\n", &q1.p4.y);

	fscanf (file_in, " %d\n", &number); /* 33 */
	fscanf (file_in, "%lf\n", &q1.p4.z);

	line_count += 24;
		
	return (q1);
}	/* read_dxf_3dface */



/************************************************************************/
struct line_seg read_dxf_line (void)
{
	int number;
	char record[32];
	struct line_seg ls1;

	printf ("Entering read_dxf_line\n");
	while (0 != strcmp ("AcDbLine", record))	{	
		/* fast forward to the important section */
		fscanf (file_in, "%s", record);
	}
	
	fscanf (file_in, " %d", &number); /* 10 */
	fscanf (file_in, "%lf", &ls1.p1.x);

	fscanf (file_in, " %d", &number); /* 20 */
	fscanf (file_in, "%lf", &ls1.p1.y);

	fscanf (file_in, " %d", &number); /* 30 */
	fscanf (file_in, "%lf", &ls1.p1.z);

	fscanf (file_in, " %d", &number); /* 11 */
	fscanf (file_in, "%lf", &ls1.p2.x);

	fscanf (file_in, " %d", &number); /* 21 */
	fscanf (file_in, "%lf", &ls1.p2.y);

	fscanf (file_in, " %d", &number); /* 31 */
	fscanf (file_in, "%lf", &ls1.p2.z);
		
	return (ls1);
}	/* read_dxf_line */



/************************************************************************/
struct point read_dxf_point (void)
{
	int number;
	char record[32];
	struct point p1;

	printf ("Entering read_dxf_point\n");
	while (0 != strcmp ("AcDbPoint", record))	{
		/* fast forward to the important section */
		fscanf (file_in, "%s", record);
	}
	
	fscanf (file_in, " %d", &number); /* 10 */
	fscanf (file_in, "%lf", &p1.x);

	fscanf (file_in, " %d", &number); /* 20 */
	fscanf (file_in, "%lf", &p1.y);

	fscanf (file_in, " %d", &number); /* 30 */
	fscanf (file_in, "%lf", &p1.z);
		
	return (p1);
}	/* read_dxf_point */

/************************************************************************/
struct arc read_dxf_arc (void)
{
	int number;
	char record[32];
	struct arc a1;

	printf ("Entering read_dxf_arc\n");
	while (0 != strcmp ("AcDbCircle", record))
	{	/* fast forward to the important section */
		fscanf (file_in, "%s", record);
	}
	
	fscanf (file_in, " %d", &number); /* 10 */
	fscanf (file_in, "%lf", &a1.p1.x);

	fscanf (file_in, " %d", &number); /* 20 */
	fscanf (file_in, "%lf", &a1.p1.y);

	fscanf (file_in, " %d", &number); /* 30 */
	fscanf (file_in, "%lf", &a1.p1.z);
		
	fscanf (file_in, " %d", &number); /* 40 */
	fscanf (file_in, "%lf", &a1.r1);
		
	fscanf (file_in, " %d", &number); /* 100 */
	fscanf (file_in, "%s", record);
		
	fscanf (file_in, " %d", &number); /* 50 */
	fscanf (file_in, "%lf", &a1.a1);
		
	fscanf (file_in, " %d", &number); /* 51 */
	fscanf (file_in, "%lf", &a1.a2);
		
	return (a1);
}	/* read_dxf_arc */

